/* This file is part of the Zenipex Library.
* Copyleft (C) 2011 Mitchell Keith Bloch a.k.a. bazald
*
* This source file is simply under the public domain.
*/

#include <zenilib.h>

#include "Tutorial_State.h"

using namespace std;
using namespace Zeni;

//#define ALTERNATIVE_RENDERER

Tutorial_State::Sound_Emitter::Sound_Emitter()
  : m_position(100.0f, 40.0f, 20.0f), 
  m_velocity(-20.0f, -10.0f, -5.0f),
  m_sound_source(new Zeni::Sound_Source(get_Sounds()["effect"],
                                        1.0f,
                                        1.0f,
                                        m_position,
                                        m_velocity,
                                        true))
{
  m_sound_source->play();
}

Tutorial_State::Sound_Emitter::~Sound_Emitter() {
  delete m_sound_source;
}

void Tutorial_State::Sound_Emitter::step(const float &time_step) {
  m_position += time_step * m_velocity;
  m_sound_source->set_position(m_position);
}

#ifdef ALTERNATIVE_RENDERER

class Volume_Renderer : public Widget_Renderer_Slider {
  void render_to(const Widget &widget) {
    const Slider * s = dynamic_cast<const Slider *>(&widget);

    if(!s)
      throw Widget_Renderer_Wrong_Type();

    Font &font = get_Fonts()["system_36_800x600"];
    string text = string("Volume").substr(0, int(6 * s->get_slider_position()));

    font.render_text(text, Point2f(s->get_end_point_a().x, s->get_end_point_a().y - font.get_text_height() / 2.0f), Color());
  }
};

Tutorial_State::BGM_Volume_Slider::BGM_Volume_Slider()
  : Slider(Zeni::Point2f(100.0f, 100.0f), Zeni::Point2f(100.0f + get_Fonts()["system_36_800x600"].get_text_width("Volume"), 100.0f),
           10.0f, 1.0f)
{
  give_Renderer(new Volume_Renderer());
}

#else

Tutorial_State::BGM_Volume_Slider::BGM_Volume_Slider()
  : Slider(Zeni::Point2f(100.0f, 100.0f), Zeni::Point2f(300.0f, 100.0f),
           10.0f, 1.0f)
{
}

#endif

void Tutorial_State::BGM_Volume_Slider::on_mouse_button(const Zeni::Point2i &pos, const bool &down) {
  Slider::on_mouse_button(pos, down, 1);
  get_Sound().set_BGM_gain(get_slider_position());
}

void Tutorial_State::BGM_Volume_Slider::on_mouse_motion(const Zeni::Point2i &pos) {
  Slider::on_mouse_motion(pos);
  get_Sound().set_BGM_gain(get_slider_position());
}

Tutorial_State::Tutorial_State()
: Widget_Gamestate(make_pair(Point2f(), Point2f(800.0f, 600.0f)), true),
m_time_passed(0.0f),
m_max_time_step(1.0f / 20.0f), // make the largest physics step 1/20 of a second
m_max_time_steps(10.0f) // allow no more than 10 physics steps per frame
{
  m_widgets.lend_Widget(m_bgm_volume_slider);

  Sound &sr = get_Sound();

  //// Set the (default) listener position, orientation, and velocity
  //sr.set_listener_position(Point3f());
  //sr.set_listener_forward_and_up(Vector3f(1.0f, 0.0f, 0.0f),
  //                               Vector3f(0.0f, 0.0f, 1.0f));
  //sr.set_listener_velocity(Vector3f());

  sr.set_BGM("music/music");
  sr.set_BGM_looping(true);
  sr.play_BGM();
}

Tutorial_State::~Tutorial_State() {
}

void Tutorial_State::perform_logic() {
  Widget_Gamestate::perform_logic();

  {// Update time_passed
    const Time temp_time = get_Timer().get_time();
    m_time_passed += temp_time.get_seconds_since(m_current_time);
    m_current_time = temp_time;
  }

  /* Simple update
   *
   * Doing more than this for Sound is pretty pointless.  In fact, 
   * I would special case Sound updates in a real game to be updated 
   * in this manner even if physics updates are done on a bounded 
   * step size.
   */
  {
    m_emitter.step(m_time_passed);
    m_time_passed = 0.0f;
  }
}
