/* This file is part of the Zenipex Library.
* Copyleft (C) 2011 Mitchell Keith Bloch a.k.a. bazald
*
* This source file is simply under the public domain.
*/

#ifndef TUTORIAL_STATE_H
#define TUTORIAL_STATE_H

#include <Zeni/Camera.h>
#include <Zeni/Game.h>
#include <Zeni/Sounds.h>
#include <Zeni/Timer.h>
#include <Zeni/Video.h>
#include <Zeni/Vector3f.h>
#include <Zeni/Widget.h>

class Tutorial_State : public Zeni::Widget_Gamestate {
  class Sound_Emitter {
  public:
    Sound_Emitter();
    ~Sound_Emitter();

    void step(const float &time_step);

  private:
    Zeni::Point3f m_position;
    Zeni::Vector3f m_velocity;

    /* Good way of thinking about things.
     * Not the best implementation in a bigger project.
     *
     * You may find that there is a limit to the number of Sound_Sources 
     * you can create in your OpenAL projects.  If this occurs, it would 
     * be a good idea to create a pool of Sound_Sources and to allow 
     * "Emitters" such as this the ability to repurpose a Sound_Source 
     * from the pool when it is time for the Sound to fire.
     *
     * Note further, that there may be NO limit.  Don't try to create 
     * as many Sound_Sources as possible until the limit is hit.  You 
     * may loop until you run out of memory...
     */
    Zeni::Sound_Source *m_sound_source;
  } m_emitter;

  class BGM_Volume_Slider : public Zeni::Slider {
  public:
    BGM_Volume_Slider();

    void on_mouse_button(const Zeni::Point2i &pos, const bool &down);
    void on_mouse_motion(const Zeni::Point2i &pos);
  } m_bgm_volume_slider;

public:
  Tutorial_State();
  ~Tutorial_State();

  void perform_logic();

private:
  Zeni::Time m_current_time;
  float m_time_passed;
  float m_max_time_step;  //< Optional
  float m_max_time_steps; //< Optional
};

#endif
